@@ -1,13 +1,11 @@
-/* This file is part of the KDE project
-   Copyright (C) 2013 Boudewijn Rempt <boud@valdyas.org>
+/*
+    This file is part of the KDE project
+    SPDX-FileCopyrightText: 2013 Boudewijn Rempt <boud@valdyas.org>
 
-   This program is free software; you can redistribute it and/or
-   modify it under the terms of the Lesser GNU General Public
-   License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 
-   This code is based on Thacher Ulrich PSD loading code released
-   on public domain. See: http://tulrich.com/geekstuff/
+    This code is based on Thacher Ulrich PSD loading code released
+    on public domain. See: http://tulrich.com/geekstuff/
 */
 
 #include "ora.h"
@@ -36,12 +34,16 @@ bool OraHandler::canRead() const
 bool OraHandler::read(QImage *image)
 {
     KZip zip(device());
-    if (!zip.open(QIODevice::ReadOnly)) return false;
+    if (!zip.open(QIODevice::ReadOnly)) {
+        return false;
+    }
 
-    const KArchiveEntry *entry = zip.directory()->entry(QLatin1String("mergedimage.png"));
-    if (!entry || !entry->isFile()) return false;
+    const KArchiveEntry *entry = zip.directory()->entry(QStringLiteral("mergedimage.png"));
+    if (!entry || !entry->isFile()) {
+        return false;
+    }
 
-    const KZipFileEntry* fileZipEntry = static_cast<const KZipFileEntry*>(entry);
+    const KZipFileEntry *fileZipEntry = static_cast<const KZipFileEntry *>(entry);
 
     image->loadFromData(fileZipEntry->data(), "PNG");
 
@@ -54,10 +56,14 @@ bool OraHandler::canRead(QIODevice *device)
         qWarning("OraHandler::canRead() called with no device");
         return false;
     }
+    if (device->isSequential()) {
+        return false;
+    }
 
     char buff[54];
-    if (device->peek(buff, sizeof(buff)) == sizeof(buff))
+    if (device->peek(buff, sizeof(buff)) == sizeof(buff)) {
         return memcmp(buff + 0x26, s_magic, s_magic_size) == 0;
+    }
 
     return false;
 }
